function [u,iflag,iter]=newton(fun,jfun,u0,params,TOLr,TOLa,maxit)
% NEWTON returns in u an approximation to the solution of
%  
%   fun(u,lambda)=0
%
%        where fun is a fuction with as meny components as u0.
%        The approximation u is found by applying Newton's method.
%
% INPUT
%   fun   function of which a zero is sought. It must be of
%         the form
%            f=fun(u,params),
%         f being a column array of same zize as u.
%         The second argument params is untouched bt NEWTON
%         and can be of any size
%   jfun  function returning the Jacobian matrix of f wrt u.
%         It must be form
%             J=jfun(u,params).
%   u0    column array with the initial guess for Newton's method.
%         Must be sufficiently close to
%   params argument to be passed to fun and jfun.
%   TOLr and TOLa scalar arguments with the values of the
%         tolerances for the absolute and relative value of
%         the error in Newton's  method: Newton's iteration is
%         stopped either when the difference between two
%         consecutive apprixmation is equal or smaller than
%               TOLr*norm(u) + TOLa
%         u being the latest approximation, or when the  number
%         of iterations exceedes maxit
%   maxit M'aximum number of iterations allowed in Neton's method.
%
% OUTPUT
%   u     computed approximation to fun(u,params)=0
%   iflag Convergence flag. If approximation u satisfies
%         
%
%         norm(e) <= TOLr*norm(u) + TOLa,    then   iflag=0
%   
%         where e=u-u_old (u_old being the computed approximation in
%         in the previous iteration). Otherwise if
%     
%         norm(e) > TOLr*norm(u) + TOLa,    then    iflag=1.
%
%  iter   number of iterations performed in Newton's method.

u=u0; iter=0; 
% we set the error to a number higher enough to ensure that at least
%  one iteration is performed
elerror=2*(TOLr*norm(u) + TOLa);
while elerror > TOLr*norm(u) + TOLa & iter < maxit
    iter=iter+1;
    f=fun(u,params); J=jfun(u,params);
    e=-(J\f); elerror=norm(e);
    u=u+e;
end
if elerror > TOLr*norm(u) + TOLa
    iflag=1;
else
    iflag=0;
end
