function [u,lambda,iflag,iter,tnew]=newton(fun,jfun,u0,params,jpar,...
               t,ur,h,TOLr,TOLa,maxit)
% NEWTON returns in u,lambda an approximation to the solution of
%        system
%  
%                    fun(u,lambda)=0,
%             t'*([u;lambda]-ur)-h=0
%
%        where fun is a fuction with as meny components as u0 and
%        lambda is params(jpar)
%        The approximation u is found by applying Newton's method.
%
% INPUT
%   fun   function of which a zero is sought. It must be of
%         the form
%            f=fun(u,params),
%         f being a column array of same zize as u.
%         The second argument params is untouched bt NEWTON
%         and can be of any size
%   jfun  function returning the Jacobian matrix of f wrt u 
%         and the partial derivativ of f wrt lambda.
%         It must be form
%             [J,fparams]=jfun(u,params).
%         returning in J the Jacobian matrix of fun wrt u, and in
%         fparams the Jacobian of f wrt the parameters params;
%   u0    column array with the initial guess for Newton's method.
%         Must be sufficiently close to
%   params argument to be passed to fun and jfun.
%   jpar  index 1<=jpar<=length(params) where parameter lambda is
%         taken.
%   ur,t  column arrays with one component more than u0 for the
%         last equation in the system
%   h     scalar for the last equation in the system
%   TOLr and TOLa scalar arguments with the values of the
%         tolerances for the absolute and relative value of
%         the error in Newton's  method: Newton's iteration is
%         stopped either when the difference between two
%         consecutive apprixmation is equal or smaller than
%               TOLr*norm(u) + TOLa
%         u being the latest approximation, or when the  number
%         of iterations exceedes maxit
%   maxit M'aximum number of iterations allowed in Neton's method.
%
% OUTPUT
%   u     computed approximation to fun(u,params)=0
%   iflag Convergence flag. If approximation u satisfies
%         
%
%         norm(e) <= TOLr*norm(u) + TOLa,    then   iflag=0
%   
%         where e=u-u_old (u_old being the computed approximation in
%         in the previous iteration). Otherwise if
%     
%         norm(e) > TOLr*norm(u) + TOLa,    then    iflag=1.
%
%  iter   number of iterations performed in Newton's method.
%  tnew   column array of same size as t with the new tangent
%         if iflag = 0 on exit and a copy of t otherwise.
%         It is botained as t=x/norm(x), where x is the solution
%         of sustem Ja*x = [zeros(size(u)); 1], Ja being the
%         coefficient matrix Newton's method.

u=u0; iter=0; 
% A copy of params in params c
paramsc=params; lambda=params(jpar); ua=[u; lambda];

% we set the error to a number higher enough to ensure that at least
%  one iteration is performed
elerror=2*(TOLr*norm(ua) + TOLa); m=length(u);
while elerror > TOLr*norm(ua) + TOLa & iter < maxit
    iter=iter+1;
    u=ua(1:m); paramsc(jpar)=ua(m+1); 
    f=fun(u,paramsc); [J,fpar]=jfun(u,paramsc);
    Ja=[J,fpar(:,jpar); t']; fa=[f; t'*(ua-ur)-h];
    ea=-(Ja\fa); elerror=norm(ea);
    ua=ua+ea;
end
u=ua(1:m); lambda=ua(m+1);
if elerror > TOLr*norm(ua) + TOLa
    iflag=1; tnew=t;
else
    iflag=0;
    % computation of the new tangent
    x=Ja\[zeros(size(u)); 1]; tnew=x/norm(x);
end
