function [x, flag, presn, it, resids] = ogmres (A, b, restart, rtol, maxit, M1,M2, x0)

  if (nargin < 2 || nargin > 8)
    print_usage ();
  end

  if (ischar (A))
    Ax = str2func (A);
  elseif (ismatrix (A))
    Ax = @(x) A*x;
  elseif (isa (A, 'function_handle'))
    Ax = A;
  else
    error ('gmres: A must be a function or matrix');
  end

  if (nargin < 3 || isempty (restart))
    restart = rows (b);
  end

  if (nargin < 4 || isempty (rtol))
    rtol = 1e-6;
  end

  if (nargin < 5 || isempty (maxit))
    maxit = min (rows (b)/restart, 10);
  end

  if (nargin < 6 || isempty (M1))
    M1m1x = @(x) x;
  elseif (ischar (M1))
    M1m1x = str2func (M1);
  elseif (ismatrix (M1))
    M1m1x = @(x) M1 \ x;
  elseif (isa (M1, 'function_handle'))
    M1m1x = M1;
  else
    error ('gmres: preconditioner M1 must be a function or matrix');
  end

  if (nargin < 7 || isempty (M2))
    M2m1x = @(x) x;
  elseif (ischar (M2))
    M2m1x = str2func (M2);
  elseif (ismatrix (M2))
    M2m1x = @(x) M2 \ x;
  elseif (isa (M2, 'function_handle'))
    M2m1x = M2;
  else
    error ('gmres: preconditioner M2 must be a function or matrix');
  end

  Pm1x = @(x) M2m1x (M1m1x (x));

  if (nargin < 8 || isempty (x0))
    x0 = zeros (size (b));
  end

  x_old = x0;
  x = x_old;
  prec_res = Pm1x (b - Ax (x_old));
  presn = norm (prec_res, 2);

  B = zeros (restart + 1, 1);
  V = zeros (rows (x), restart);
  H = zeros (restart + 1, restart);

  % begin loop
  iter = 1;
  restart_it  = restart + 1;
  resids      = zeros (maxit, 1);
  resids(1)   = presn;
  prec_b_norm = norm (Pm1x (b), 2);
  flag        = 1;
  stag=0; maxstag=4;

  while (iter <= maxit * restart && presn > rtol * prec_b_norm)

    % restart
    if (restart_it > restart)
      restart_it = 1;
      x_old = x;
      prec_res = Pm1x (b - Ax (x_old));
      presn = norm (prec_res, 2);
      B(1) = presn;
      H(:) = 0;
      V(:, 1) = prec_res / presn;
    end

    % basic iteration
    tmp = Pm1x (Ax (V(:, restart_it)));
    [V(:,restart_it+1), H(1:restart_it+1, restart_it)] = ...
        mgorth (tmp, V(:,1:restart_it));

    Y = (H(1:restart_it+1, 1:restart_it) \ B (1:restart_it+1));

    little_res = B(1:restart_it+1) - ...
        H(1:restart_it+1, 1:restart_it) * Y(1:restart_it);

    presn = norm (little_res, 2); 

    x = x_old + V(:, 1:restart_it) * Y(1:restart_it);

    resids(iter) = presn;
    if (norm (x - x_old, inf) <= eps)
        stag=stag+1;
    else
        stag=0;
    end
    if stag>=maxstag
      flag = 3;
      break
    end

    restart_it=restart_it+1 ;
    iter=iter+1;
  endwhile

  if (presn <= rtol * prec_b_norm)
    flag = 0;
  end

  resids = resids(1:iter-1);
  it = [ceil(iter / restart), rem(iter, restart)];

  end


%!shared A, b, dim
%! dim = 100;
%!test
%! A = spdiags ([-ones(dim,1) 2*ones(dim,1) ones(dim,1)], [-1:1], dim, dim);
%! b = ones(dim, 1);
%! x = gmres (A, b, 10, 1e-10, dim, @(x) x./diag(A), [],  b);
%! assert(x, A\b, 1e-9*norm(x,inf));
%!
%!test
%! x = gmres (A, b, dim, 1e-10, 1e4, @(x) diag(diag(A))\x, [],  b);
%! assert(x, A\b, 1e-7*norm(x,inf));
%!
%!test
%! A = spdiags ([[1./(2:2:2*(dim-1)) 0]; 1./(1:2:2*dim-1); [0 1./(2:2:2*(dim-1))]]', -1:1, dim, dim);
%! A = A'*A;
%! b = rand (dim, 1);
%! [x, resids] = gmres (@(x) A*x, b, dim, 1e-10, dim, @(x) x./diag(A), [],  []);
%! assert(x, A\b, 1e-9*norm(x,inf))
%! x = gmres (@(x) A*x, b, dim, 1e-10, 1e6, @(x) diag(diag(A))\x, [],  []);
%! assert(x, A\b, 1e-9*norm(x,inf));
%!test
%! x =  gmres (@(x) A*x, b, dim, 1e-10, 1e6, @(x) x./diag(A), [],  []);
%! assert(x, A\b, 1e-7*norm(x,inf));